"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const boom_1 = __importDefault(require("@hapi/boom"));
const class_validator_1 = require("class-validator");
const search_term_internal_1 = __importDefault(require("../models/internal/search-term.internal"));
class BaseService {
    constructor(repository) {
        this.repository = repository;
    }
    preSaveHook(entity) {
    }
    preUpdateHook(entity) {
    }
    preDeleteHook(entity) {
    }
    preResultHook(entity) {
    }
    validId(id) {
        return id !== undefined && id > 0;
    }
    isValid(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const errors = yield class_validator_1.validate(entity, {
                    validationError: { target: false, value: false }
                });
                if (errors.length > 0) {
                    throw boom_1.default.badRequest('Validation failed on the provided request', errors);
                }
                return true;
            }
            catch (error) {
                if (boom_1.default.isBoom(error)) {
                    throw boom_1.default.boomify(error);
                }
                throw boom_1.default.badRequest('Unable to validate request: ' + error);
            }
        });
    }
    findAll() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entities = yield this.repository.getAll();
                entities.map(item => this.preResultHook(item));
                return entities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    findAllByFilter(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entities = yield this.repository.findManyByFilter(filter);
                entities.map(item => this.preResultHook(item));
                return entities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    findOneById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.validId(id) || isNaN(id)) {
                    throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                }
                const entity = yield this.repository.findOneById(id);
                this.preResultHook(entity);
                return entity;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    findOneByFilter(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entityResult = yield this.repository.findOneByFilter(filter);
                this.preResultHook(entityResult);
                return entityResult;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    findOneWithQueryBuilder(options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entityResult = yield this.repository.findOneWithQueryBuilder(options);
                if (entityResult) {
                    this.preResultHook(entityResult);
                    return entityResult;
                }
                else {
                    throw boom_1.default.notFound('The requested object could not be found');
                }
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    findManyWithQueryBuilder(options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entities = yield this.repository.findManyWithQueryBuilder(options);
                entities.map(item => this.preResultHook(item));
                return entities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    search(limit, searchTerms) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const filter = this.getSearchFilter(limit, searchTerms);
                const entities = yield this.findManyWithQueryBuilder(filter);
                entities.map(item => this.preResultHook(item));
                return entities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    save(entity) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entityIsValid = yield this.isValid(entity);
                if (!entityIsValid) {
                    throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                }
                this.preSaveHook(entity);
                const savedEntity = yield this.repository.save(entity);
                this.preResultHook(savedEntity);
                return savedEntity;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    saveAll(entities) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (const entity of entities) {
                    const entityIsValid = yield this.isValid(entity);
                    if (!entityIsValid) {
                        throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                    }
                    this.preSaveHook(entity);
                }
                const savedEntities = yield this.repository.saveAll(entities);
                savedEntities.map(item => this.preResultHook(item));
                return savedEntities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    update(entity, id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const entityIsValid = yield this.isValid(entity);
                if (!entityIsValid || !this.validId(id)) {
                    throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                }
                this.preUpdateHook(entity);
                const updatedEntity = yield this.repository.updateOneById(id, entity);
                this.preResultHook(updatedEntity);
                return updatedEntity;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    updateAll(entities) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (const entity of entities) {
                    const entityIsValid = yield this.isValid(entity);
                    if (!entityIsValid) {
                        throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                    }
                    this.preUpdateHook(entity);
                }
                const updatedEntities = yield this.repository.updateAll(entities);
                updatedEntities.map(item => this.preResultHook(item));
                return updatedEntities;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    delete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.validId(id)) {
                    throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                }
                const entityResult = yield this.repository.findOneById(id);
                this.preDeleteHook(entityResult);
                const deletedEntity = yield this.repository.delete(entityResult);
                this.preResultHook(deletedEntity);
                return deletedEntity;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    softDelete(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.validId(id)) {
                    throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
                }
                const entityResult = yield this.repository.findOneById(id);
                this.preDeleteHook(entityResult);
                const deletedEntity = yield this.repository.save(entityResult);
                this.preResultHook(deletedEntity);
                return deletedEntity;
            }
            catch (error) {
                if (error && error.isBoom) {
                    throw error;
                }
                throw boom_1.default.internal(error);
            }
        });
    }
    getSearchFilter(limit, searchTerms) {
        if (limit >= 0 && searchTerms && searchTerms.length > 0) {
            let whereClause = '';
            const andWhereClause = [];
            for (const searchTerm of searchTerms) {
                const term = search_term_internal_1.default.newSearchTerm(searchTerm);
                let quoteValue = true;
                if (searchTerm.value.startsWith('(') && searchTerm.value.endsWith(')')) {
                    quoteValue = false;
                }
                const value = quoteValue ? `'${term.value}'` : `${term.value}`;
                if (!whereClause || whereClause === '') {
                    whereClause = `${term.field} ${term.operator ? term.operator : ' = '} ${value}`;
                }
                else {
                    andWhereClause.push(`${term.field} ${term.operator ? term.operator : ' = '} ${value}`);
                }
            }
            return {
                where: whereClause,
                andWhere: andWhereClause,
                limit
            };
        }
        else {
            throw boom_1.default.badRequest('Incorrect / invalid parameters supplied');
        }
    }
}
exports.default = BaseService;
